﻿IM = InterruptManager
IM.rotation = {}

function IM.GetVersion() return 127 end
local interruptSpells = IM.GetInterruptSpells()

function IM.SavePosition()
    InterruptManagerAnchor:StopMovingOrSizing()
    for i=1,5 do
        IMDB.anchorPoint[i] = select(i,InterruptManagerAnchor:GetPoint())
    end
end

--[[    IMDB = {
                [1] = Anchor:GetPoint()
                [2] = Anchor:GetSize()
                [3] = Anchor:GetMovable()
                [4] = Anchor:GetAlpha()
                [5] = Icon:GetPoint(x,y)
                [6] = Icon:GetSize()
                [7] = Icon:GetAlpha()
                [8] = Icon.text:GetFontSize(),Icon.text:GetFontAlpha()
                [9] = StatusBar:GetSize()
                [10]= StatusBar:GetPoint(x,y)
                [11]= StatusBar:GetAlpha()
                [12]= StatusBar.text:GetSize(),StatusBar.text:GetAlpha()
                [13]= SoloMode
                [14]= Version
                [15]= Announce
                [16]= PUGmode
                [17]= TargetWarn
                [18]= FocusWarn
                [19]= AnnounceAfterPUGmodeOff
                }
]]

function IM.AssignSavedVariables()
    if not IMDB then
        IMDB = {}
        if not IMDB.anchorPoint then
            IMDB =  {
                    ["anchorPoint"] = {"CENTER",nil,"CENTER",0,0},
                    ["anchorSize"] = {180,100},
                    ["anchorMovable"] = 1,
                    ["anchorAlpha"] = 0.2,
                    ["iconPoint"] = {-20,20},
                    ["iconSize"] = {20,20},
                    ["iconAlpha"] = 0.4,
                    ["iconTextSize"] = {12,1},
                    ["statusBarSize"] = {180,20},
                    ["statusBarPoint"]= {0,0},
                    ["statusBarAlpha"]= 1,
                    ["statusBarTextSize"]= {12,1}
                    }
        end
        IMDB.soloMode = 0
        IMDB.version = IM.GetVersion()
        IMDB.announce = 0
        IMDB.pugMode = 0
        IMDB.targetWarn = 1
        IMDB.focusWarn = 0
        IMDB.announceAfterPugModeOff = 0
    else
        IMDB.pugMode = 0
    end
end

function IM.InitializeInterrupterTable()
    IMDB.interrupters = {}
    
    for i = 1,5 do
        IMDB.interrupters[i] = {}
        IMDB.interrupters[i].active = false
        IMDB.interrupters[i].spells = {}
        IMDB.interrupters[i].name = ""
        IMDB.interrupters[i].ready = false
        IMDB.interrupters[i].next = false
        IMDB.interrupters[i].pos = i
        IMDB.interrupters[i].readyTime = i
        IMDB.interrupters[i].lastInterrupt = GetTime()
        
        IM.rotation[i] = ""
    end
end

function IM.VariablesLoaded()
    if (not IMDB) then
        IM.AssignSavedVariables()
    elseif (IMversion ~= IMDB.version) then
        IM.AssignSavedVariables()
    end
    
    if (not IMDB.interrupters or IMDB.version ~= IM.GetVersion()) then
        IM.InitializeInterrupterTable()
    end
    
    IMDB.version = IM.GetVersion()
    
    -- Create the main frame
    local f = CreateFrame("Frame","InterruptManagerAnchor")
    f:SetSize(IMDB.anchorSize[1],IMDB.anchorSize[2])
    f:SetPoint(IMDB.anchorPoint[1],IMDB.anchorPoint[2],IMDB.anchorPoint[3],IMDB.anchorPoint[4],IMDB.anchorPoint[5])
    f:SetParent(UIParent)
    if IMDB.anchorMovable == 1 then
        f:SetMovable(IMDB.anchorMovable)
        f:SetScript("OnMouseDown",function()f:StartMoving()end)
        f:SetScript("OnMouseUp",IM.SavePosition)
    end
    local t = f:CreateTexture()
    t:SetTexture(0,0,0,IMDB.anchorAlpha)
    t:SetAllPoints(f)
    
    -- Create each individual bar
    for i = 1,5 do
        local f = CreateFrame("Frame","InterruptManagerIcon"..i)
        f:SetSize(IMDB.iconSize[1],IMDB.iconSize[2])
        f:SetPoint("TOPLEFT","InterruptManagerAnchor","TOPLEFT",IMDB.iconPoint[1],-IMDB.iconPoint[2]*i+IMDB.iconPoint[2])
        f:SetParent(InterruptManagerAnchor)
        local t = f:CreateTexture()
        t:SetAllPoints(f)
        t:SetTexture(0,0,0,IMDB.iconAlpha)
        t = f:CreateFontString()
        t:SetPoint("CENTER","InterruptManagerIcon"..i,"CENTER")
        t:SetFont("Fonts\\FRIZQT__.TTF", IMDB.iconTextSize[1])
        t:SetText(i)
        t:SetTextColor(1,1,1,IMDB.iconTextSize[2])
    
        f = CreateFrame("StatusBar", "InterruptManagerStatusBar"..i)
        f:SetSize(IMDB.statusBarSize[1],IMDB.statusBarSize[2])
        f:SetPoint("LEFT","InterruptManagerIcon"..i,"RIGHT",IMDB.statusBarPoint[1],IMDB.statusBarPoint[2])
        f:SetParent(InterruptManagerAnchor)
        f:SetOrientation("HORIZONTAL")
        f:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
        f:SetStatusBarColor(0,1,0,IMDB.statusBarAlpha)
        f:SetFrameLevel(3)
        f:SetMinMaxValues(0,1)
        f:SetValue(0)
        t = f:CreateFontString("InterruptManagerStatusBarText"..i)
        t:SetPoint("LEFT","InterruptManagerStatusBar"..i,"LEFT",5,0)
        t:SetFont("Fonts\\FRIZQT__.TTF",IMDB.statusBarTextSize[1])
        t:SetTextColor(1,1,1,IMDB.statusBarTextSize[2])
        t:SetText(IMDB.interrupters[i].name)
        
        local g = CreateFrame("StatusBar", "InterruptManagerSecondaryStatusBar"..i)
        g:SetSize(IMDB.statusBarSize[1],IMDB.statusBarSize[2])
        g:SetPoint("CENTER",f,"CENTER")
        g:SetParent(f)
        g:SetOrientation("HORIZONTAL")
        g:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
        g:SetStatusBarColor(1,0,0,IMDB.statusBarAlpha/2)
        g:SetFrameLevel(2)
        g:SetMinMaxValues(0,1)
        g:SetValue(0)
    end
    
    -- Create the mid-screen warning text frame
    local c = CreateFrame("MessageFrame","InterruptManagerText")
    c:SetWidth(300)
    c:SetHeight(50)
    c:SetPoint("CENTER", UIParent, "CENTER", 0, 80)
    c:SetFont("Fonts\\FRIZQT__.TTF", 40)
    c:SetFadeDuration(1)
    c:SetTimeVisible(2)
    IM.OnLogin()
    f:SetScript("OnUpdate",IM.OnUpdate)
    RegisterAddonMessagePrefix("InterruptManager")
end

function IM.OnUpdate()
    for k,v in pairs(IMDB.interrupters) do
        if (v.active) then
            -- Update StatusBar values
            _G["InterruptManagerStatusBar"..k]:SetValue(v.readyTime - GetTime())
            
            -- Update text to reflect cooldown
            if (v.readyTime - GetTime() > 0) then
                _G["InterruptManagerStatusBarText"..k]:SetText(v.name .. " " .. round(v.readyTime - GetTime(),1))
            
            -- Update text to only show name when cooldown is finished
            elseif (_G["InterruptManagerStatusBarText"..k]:GetText() ~= v.name) then
                _G["InterruptManagerStatusBarText"..k]:SetText(v.name)
                
            end
            
            for k2,v2 in pairs(v.spells) do
                if (v2 - GetTime() > 0) then
                    if (v.readyTime - GetTime() > 0) then
                        _G["InterruptManagerStatusBarText"..k]:SetText(v.name .. " " .. round(v.readyTime - GetTime(),1) .. " (" .. round(v2 - GetTime(),1) .. ")")
                    else
                        _G["InterruptManagerStatusBarText"..k]:SetText(v.name .. " (" .. round(v2 - GetTime(),1) .. ")")
                    end
                    
                    _G["InterruptManagerSecondaryStatusBar"..k]:SetValue(v2 - GetTime())
                end
            end
        end
    end
end

function IM.OnEvent(self, event, ...)
    if event == "COMBAT_LOG_EVENT_UNFILTERED" then
        IM.CombatLogEvent(self, event, ...)
    elseif event == "VARIABLES_LOADED" then
        IM.VariablesLoaded()
    elseif event == "PARTY_MEMBERS_CHANGED" then
    elseif event == "CHAT_MSG_ADDON" and ... == "InterruptManager" then
        IM.AddonMessageReceived(...)
    elseif event == "CHAT_MSG_SYSTEM" then
        IM.ChatMsgSystem(self, event, ...)
    elseif event == "PLAYER_REGEN_DISABLED" then
        IM.EnterCombat()
    end
end

function IM.EnterCombat()
    if IMDB.pugMode == 0 then return end
    
    for i = 1,5 do
        if _G["InterruptManagerStatusBarText"..i]:GetText() then
            SendChatMessage(i..": ".._G["InterruptManagerStatusBarText"..i]:GetText(),"SAY")
        end
    end
    if InterruptManagerStatusBarText1:GetText() then
        SendChatMessage("Interrupting next: " .. InterruptManagerStatusBarText1:GetText())
    end
end

f = CreateFrame("Frame","InterruptManager222")
f:SetScript("OnEvent",IM.OnEvent)
f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
f:RegisterEvent("VARIABLES_LOADED")
f:RegisterEvent("PARTY_MEMBERS_CHANGED")
f:RegisterEvent("CHAT_MSG_ADDON")
f:RegisterEvent("CHAT_MSG_SYSTEM")
f:RegisterEvent("PLAYER_REGEN_DISABLED")

function IM.CastWarning(event, notInterruptible, sourceGUID, sourceName, spellId, spellName)
    -- Warning when target starts casting
    if UnitExists("target") and event == "SPELL_CAST_START" and sourceGUID == UnitGUID("target") and not notInterruptible and IMDB.targetWarn == 1 then
        if IMDB.soloMode == 0 then
            for i = 1,5 do
                if select(2,getglobal("InterruptManagerStatusBar"..i):GetPoint()) == InterruptManagerIcon1 then
                    if getglobal("InterruptManagerStatusBarText"..i):GetText() == UnitName("player") then
                        InterruptManagerText:AddMessage("Interrupt now! (target)",0,0,1)
                        PlaySoundFile("Sound\\Interface\\PVPFlagTakenMono.wav")
                    end
                    break
                end
            end
        else
            InterruptManagerText:AddMessage("Interrupt now! (target)",0,0,1)
            PlaySoundFile("Sound\\Interface\\PVPFlagTakenMono.wav")
        end
        
    -- Warning when focus starts casting
    elseif UnitExists("focus") and event == "SPELL_CAST_START" and sourceGUID == UnitGUID("focus") and not notInterruptible and IMDB.focusWarn == 1 then
        if IMDB.soloMode == 0 then
            for i = 1,5 do
                if select(2,getglobal("InterruptManagerStatusBar"..i):GetPoint()) == InterruptManagerIcon1 then
                    if getglobal("InterruptManagerStatusBarText"..i):GetText() == UnitName("player") then
                        InterruptManagerText:AddMessage("Interrupt now! (focus)",0.3,0,1)
                        PlaySoundFile("Sound\\Interface\\PVPFlagTakenMono.wav")
                    end
                    break
                end
            end
        else
            InterruptManagerText:AddMessage("Interrupt now! (focus)",0,0.3,1)
            PlaySoundFile("Sound\\Interface\\PVPFlagTakenMono.wav")
        end
    end
end

function IM.UpdateInterruptRotation()
    local db = IMDB.interrupters
    
    local numInterrupters = 0
    for i = 1,5 do
        if (db[i].active) then
            numInterrupters = numInterrupters + 1
        end
    end
    
    -- Sort interrupters, player with the lowest readyTime first
    local t = {} -- temporary table
    local r = IM.rotation
    wipe(r)
    
    for i = 1,numInterrupters do
        tinsert(t, db[i].readyTime)
    end
    sort(t)
    
    for i = 1,#t do
        for j = 1,#t do
            if (t[i] == db[j].readyTime and not tContains(r, db[j].name)) then
                r[i] = db[j].name
            end
        end
    end
    
    IM.UpdateStatusBarPositions()
end

function IM.UpdateStatusBarPositions()
    for k,v in pairs(IM.rotation) do
        IM.SetNameToPosition(v,k)
    end
end

function IM.CombatLogEvent(self, event, ...)
    local event = select(2, ...)
    local notInterruptible = select(9, UnitCastingInfo("target"))
    local sourceGUID = select(4, ...)
    local sourceName = select(5, ...)
    local spellId = select(12, ...)
    local spellName = select(13, ...)
    local cooldown
    local db = IMDB.interrupters
    
    IM.CastWarning(event, notInterruptible, sourceGUID, sourceName, spellId, spellName)
    
    -- Fires when someone in my rotation uses an interrupt spell
    if event == "SPELL_CAST_SUCCESS" then
        for i = 1,5 do
            if (sourceName == db[i].name) then
                for k,v in pairs(interruptSpells.spellId) do
                    if (v == spellId) then

                        if (not IM.IsSecondarySpell(v)) then
                            _G["InterruptManagerStatusBar"..i]:SetMinMaxValues(0, interruptSpells.cooldown[k])
                            db[i].readyTime = GetTime() + interruptSpells.cooldown[k]
                            IM.UpdateInterruptRotation()
                        else
                            _G["InterruptManagerSecondaryStatusBar"..i]:SetMinMaxValues(0, interruptSpells.cooldown[k])
                            db[i].spells[v] = GetTime() + interruptSpells.cooldown[k]
                        end
                    end
                end
            end
        end
    end
end

-- [Function to prevent system message spam] --
ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", function(self, event, msg)
    if(strfind(msg, "No player named ")) then
        return true
    end
end)
local AntiSpam = {{},{}}
function IM.ChatMsgSystem(self, event, ...)
    local text = ...
    if strfind(text, "No player named ") then
        if not tContains(AntiSpam[1],text) then
            DEFAULT_CHAT_FRAME:AddMessage(text,1,1,0)
            tinsert(AntiSpam[1],text)
            tinsert(AntiSpam[2],GetTime()+1)
            else
            for k,v in pairs(AntiSpam[1]) do
                if v == text then
                    if GetTime() >= AntiSpam[2][k] then
                        AntiSpam[2][k] = GetTime()+1
                        DEFAULT_CHAT_FRAME:AddMessage(text,1,1,0)
                    end
                end
            end
        end
    end
end

SLASH_INTERRUPTMANAGER1 = '/im'
SLASH_INTERRUPTMANAGER2 = '/interruptmanager'
function SlashCmdList.INTERRUPTMANAGER(msg, editbox)
    if string.find(msg, "lock") then
        IM.OnFrameLock()
        else
        IM.OpenConfig()
    end
end